"""Module init."""
import importlib.metadata

from hume.batch import BatchJob, BatchJobResult, BatchJobStatus, HumeBatchClient
from hume.common.hume_client_error import HumeClientError
from hume.common.model_type import ModelType

__version__ = importlib.metadata.version("hume")

__all__ = [
    "__version__",
    "BatchJob",
    "BatchJobResult",
    "BatchJobStatus",
    "HumeBatchClient",
    "HumeClientError",
    "ModelType",
]



"""Configuration for the facial expression model."""
from typing import Any, Dict, Optional

from hume.common.config.job_config_base import JobConfigBase
from hume.common.model_type import ModelType


class FaceConfig(JobConfigBase["FaceConfig"]):
    """Configuration for the facial expression model."""

    def __init__(
        self,
        fps_pred: Optional[float] = None,
        prob_threshold: Optional[float] = None,
        identify_faces: Optional[bool] = None,
        min_face_size: Optional[float] = None,
    ):
        """Construct a `FaceConfig`.

        Args:
            fps_pred (Optional[float]): Number of frames per second to process. Other frames will be omitted
                from the response.
            prob_threshold (Optional[float]): Face detection probability threshold. Faces detected with a
                probability less than this threshold will be omitted from the response.
            identify_faces (Optional[bool]): Whether to return identifiers for faces across frames.
                If true, unique identifiers will be assigned to face bounding boxes to differentiate different faces.
                If false, all faces will be tagged with an "unknown" ID.
            min_face_size (Optional[float]): Minimum bounding box side length in pixels to treat as a face.
                Faces detected with a bounding box side length in pixels less than this threshold will be
                omitted from the response.
        """
        self.fps_pred = fps_pred
        self.prob_threshold = prob_threshold
        self.identify_faces = identify_faces
        self.min_face_size = min_face_size

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.FACE

    def serialize(self) -> Dict[str, Any]:
        """Serialize `FaceConfig` to dictionary.

        Returns:
            Dict[str, Any]: Serialized `FaceConfig` object.
        """
        return {
            "fps_pred": self.fps_pred,
            "prob_threshold": self.prob_threshold,
            "identify_faces": self.identify_faces,
            "min_face_size": self.min_face_size,
        }

    @classmethod
    def deserialize(cls, request_dict: Dict[str, Any]) -> "FaceConfig":
        """Deserialize `FaceConfig` from request JSON.

        Args:
            request_dict (Dict[str, Any]): Request JSON data.

        Returns:
            FaceConfig: Deserialized `FaceConfig` object.
        """
        return cls(
            fps_pred=request_dict["fps_pred"],
            prob_threshold=request_dict["prob_threshold"],
            identify_faces=request_dict["identify_faces"],
            min_face_size=request_dict["min_face_size"],
        )



"""Model configuration utilities."""
from typing import Type

from hume.common.model_type import ModelType
from hume.common.config import BurstConfig, FaceConfig, LanguageConfig, JobConfigBase, ProsodyConfig
from hume.common.hume_client_error import HumeClientError


def config_from_model_type(model_type: ModelType) -> Type[JobConfigBase]:
    """Get the configuration type for the given model type.

    Args:
        model_type (ModelType): Model type of configuration.

    Returns:
        Type[JobConfigBase]: Class of configuration for the given model type.
    """
    if model_type == ModelType.BURST:
        return BurstConfig
    if model_type == ModelType.FACE:
        return FaceConfig
    if model_type == ModelType.LANGUAGE:
        return LanguageConfig
    if model_type == ModelType.PROSODY:
        return ProsodyConfig
    raise HumeClientError(f"Unknown model type {model_type}")



"""Function retry utilities."""
import logging
import time
from typing import cast, Callable, Type, TypeVar
from typing_extensions import ParamSpec

from hume.common.hume_client_error import HumeClientError

P = ParamSpec('P')  # Parameter type variable for decorated function
R = TypeVar('R')  # Return type variable for decorated function

logger = logging.getLogger(__name__)


class RetryIterError(Exception):
    """Retry iteration exception.

    Raised when a job has not completed by the allotted timeout.
    """


def retry(
    timeout: int = 300,
    backoff_factor: int = 2,
    error_type: Type[Exception] = RetryIterError,
) -> Callable[[Callable[P, R]], Callable[P, R]]:
    """Retry decorator for exponential backoff retry.

    Args:
        timeout (int): Maximum seconds to keep retrying before giving up. Defaults to 300.
        backoff_factor (int): Multiplier factor for exponential backoff. Defaults to 2.
        error_type (Type[Exception]): Class of exception to expect from decorated function when
            the function fails. Raise this exception type if the retry iteration has failed.
            Defaults to RetryIterError.

    Returns:
        Callable[[Callable[P, R]], Callable[P, R]]: Function decorator.
    """

    def decorator_func(decorated_func: Callable[P, R]) -> Callable[P, R]:

        def func_wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
            inner_timeout = timeout
            if "timeout" in kwargs:
                inner_timeout = cast(int, kwargs["timeout"])
            inner_backoff_factor = backoff_factor
            if "backoff_factor" in kwargs:
                inner_backoff_factor = cast(int, kwargs["backoff_factor"])

            delay = 1
            total_await_time = 0
            attempt = 1

            while True:
                logger.info(f"Retry attempt {attempt}, waited {total_await_time}s total")

                try:
                    return decorated_func(*args, **kwargs)
                except error_type as e:
                    logger.info(f"Retry iteration {attempt} failed: {str(e)}")

                if total_await_time >= inner_timeout:
                    raise HumeClientError(f"Request timed out after {inner_timeout}s")

                time.sleep(delay)
                total_await_time += delay

                new_delay = delay * inner_backoff_factor
                if total_await_time + new_delay > inner_timeout:
                    delay = inner_timeout - total_await_time
                else:
                    delay = new_delay

                attempt += 1

        return func_wrapper

    return decorator_func



"""Batch job."""
import logging
from typing import TYPE_CHECKING

from hume.batch.batch_job_result import BatchJobResult
from hume.batch.batch_job_status import BatchJobStatus
from hume.common.retry_utils import retry, RetryIterError

if TYPE_CHECKING:
    from hume.batch.hume_batch_client import HumeBatchClient

logger = logging.getLogger(__name__)


class BatchJob:
    """Batch job."""

    def __init__(self, client: "HumeBatchClient", job_id: str):
        """Construct a BatchJob.

        Args:
            client (HumeBatchClient): HumeBatchClient instance.
            job_id (str): Job ID.
        """
        self._client = client
        self.id = job_id

    def await_complete(self, timeout: int = 300) -> BatchJobResult:
        """Block until the job has reached a terminal status.

        Args:
            timeout (int): Maximum time in seconds to await before failing.
                Will fail with `HumeClientError` if timeout is reached.

        Raises:
            ValueError: If the timeout is not valid.

        Returns:
            BatchJobResult: The result of the `BatchJob`.
        """
        if timeout < 1:
            raise ValueError("timeout must be at least 1 second")

        return self._await_complete(timeout=timeout)

    # pylint: disable=unused-argument
    @retry()
    def _await_complete(self, timeout: int = 300) -> BatchJobResult:
        result = self._client.get_job_result(self.id)
        if not BatchJobStatus.is_terminal(result.status):
            raise RetryIterError
        return result

    def __repr__(self) -> str:
        """Get the string representation of the `BatchJob`.

        Returns:
            The the string representation of the `BatchJob`.
        """
        return f'Job(id="{self.id}")'



"""Module init."""
from hume.common.config.burst_config import BurstConfig
from hume.common.config.face_config import FaceConfig
from hume.common.config.language_config import LanguageConfig
from hume.common.config.job_config_base import JobConfigBase
from hume.common.config.prosody_config import ProsodyConfig

__all__ = [
    "BurstConfig",
    "FaceConfig",
    "LanguageConfig",
    "JobConfigBase",
    "ProsodyConfig",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hume', 'hume.batch', 'hume.common', 'hume.common.config']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=2.1.2,<3.0.0',
 'requests>=2.26.0,<3.0.0',
 'typing-extensions>=4.3.0,<5.0.0']

setup_kwargs = {
    'name': 'hume',
    'version': '0.1.0',
    'description': 'Hume AI Python Client',
    'long_description': '# Hume AI Python Client\n\nThe Hume AI Python client makes it easy to call Hume APIs from Python applications.\n\nTo get started, [sign up for a Hume account](https://beta.hume.ai/sign-up)!\n\n## Requirements\n\nPython versions between 3.8 and 3.10 are supported\n\n## Installation\n\n```\npip install hume\n```\n\n## Example Usage\n\n### Submit a new batch job\n\n> Note: Your personal API key can be found in the profile section of [beta.hume.ai](https://beta.hume.ai)\n\n```python\nfrom hume import HumeBatchClient\n\nclient = HumeBatchClient("<your-api-key>")\nurls = ["https://tinyurl.com/hume-img"]\njob = client.submit_face(urls)\n\nprint(job)\nprint("Running...")\n\nresult = job.await_complete()\nresult.download_predictions("predictions.json")\n\nprint("Predictions downloaded!")\n```\n\n### Rehydrate a batch job from a job ID\n\n```python\nfrom hume import BatchJob, HumeBatchClient\n\nclient = HumeBatchClient("<your-api-key>")\n\njob_id = "<your-job-id>"\njob = BatchJob(client, job_id)\n\nprint(job)\n```\n\n## Documentation\n\nLearn more about Hume\'s expressive communication platform on [our homepage](https://hume.ai) or our [platform docs](https://help.hume.ai/basics/about-hume-ai)\n\nSee example requests and responses for all available endpoints in the [Hume API Reference](https://docs.hume.ai)\n\n## Support\n\nIf you\'ve found a bug with this client please [open an issue](https://github.com/HumeAI/python-client/issues/new)!\n',
    'author': 'Hume AI Dev',
    'author_email': 'dev@hume.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/HumeAI/python-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)



"""Batch API client."""
import json
import logging
from typing import Any, Dict, List, Optional

import requests

from hume.batch.batch_job import BatchJob
from hume.batch.batch_job_result import BatchJobResult
from hume.common.config import BurstConfig, FaceConfig, LanguageConfig, ProsodyConfig, JobConfigBase
from hume.common.api_type import ApiType
from hume.common.client_base import ClientBase
from hume.common.hume_client_error import HumeClientError

logger = logging.getLogger(__name__)


class HumeBatchClient(ClientBase):
    """Batch API client."""

    def __init__(self, *args: Any, **kwargs: Any):
        """Construct a HumeBatchClient.

        Args:
            api_key (str): Hume API key.
        """
        super().__init__(*args, **kwargs)

    def get_job_result(self, job_id: str) -> BatchJobResult:
        """Get the result of the batch job.

        Args:
            job_id (str): Job ID.

        Raises:
            HumeClientError: If the job result cannot be loaded.

        Returns:
            BatchJobResult: Batch job result.
        """
        endpoint = (f"{self._api_base_url}/{self._api_version}/{ApiType.BATCH.value}/jobs/{job_id}"
                    f"?apikey={self._api_key}")
        response = requests.get(endpoint)
        body = response.json()
        return BatchJobResult.from_response(body)

    def submit_face(
        self,
        urls: List[str],
        fps_pred: Optional[float] = None,
        prob_threshold: Optional[float] = None,
        identify_faces: Optional[bool] = None,
        min_face_size: Optional[float] = None,
    ) -> BatchJob:
        """Submit a new job for facial expression.

        Args:
            urls (List[str]): URLs to process.
            fps_pred (Optional[float]): Number of frames per second to process. Other frames will be omitted
                from the response.
            prob_threshold (Optional[float]): Face detection probability threshold. Faces detected with a
                probability less than this threshold will be omitted from the response.
            identify_faces (Optional[bool]): Whether to return identifiers for faces across frames.
                If true, unique identifiers will be assigned to face bounding boxes to differentiate different faces.
                If false, all faces will be tagged with an "unknown" ID.
            min_face_size (Optional[float]): Minimum bounding box side length in pixels to treat as a face.
                Faces detected with a bounding box side length in pixels less than this threshold will be
                omitted from the response.

        Raises:
            HumeClientError: If the job fails.

        Returns:
            BatchJob: Batch job.
        """
        config = FaceConfig(
            fps_pred=fps_pred,
            prob_threshold=prob_threshold,
            identify_faces=identify_faces,
            min_face_size=min_face_size,
        )
        return self._submit(urls, [config])

    def submit_burst(
        self,
        urls: List[str],
    ) -> BatchJob:
        """Submit a new job for vocal bursts.

        Args:
            urls (List[str]): URLs to process.

        Raises:
            HumeClientError: If the job fails.

        Returns:
            BatchJob: Batch job.
        """
        config = BurstConfig()
        return self._submit(urls, [config])

    def submit_prosody(
        self,
        urls: List[str],
        identify_speakers: Optional[bool] = None,
    ) -> BatchJob:
        """Submit a new job for vocal bursts.

        Args:
            urls (List[str]): URLs to process.
            identify_speakers (Optional[bool]): Whether to return identifiers for speakers over time. If true,
                unique identifiers will be assigned to spoken words to differentiate different speakers. If false,
                all speakers will be tagged with an "unknown" ID.

        Raises:
            HumeClientError: If the job fails.

        Returns:
            BatchJob: Batch job.
        """
        config = ProsodyConfig(identify_speakers=identify_speakers)
        return self._submit(urls, [config])

    def submit_language(
        self,
        urls: List[str],
        sliding_window: Optional[bool] = None,
        identify_speakers: Optional[bool] = None,
    ) -> BatchJob:
        """Submit a new job for language emotion.

        Args:
            urls (List[str]): URLs to process.
            sliding_window (Optional[float]): Whether to generate predictions for each word in the text or
                for the entire text in aggregate.
            identify_speakers (Optional[bool]): Whether to return identifiers for speakers over time.
                If true, unique identifiers will be assigned to spoken words to differentiate different speakers.
                If false, all speakers will be tagged with an "unknown" ID.

        Raises:
            HumeClientError: If the job fails.

        Returns:
            BatchJob: Batch job.
        """
        config = LanguageConfig(
            sliding_window=sliding_window,
            identify_speakers=identify_speakers,
        )
        return self._submit(urls, [config])

    def _submit(self, urls: List[str], configs: List[JobConfigBase]) -> BatchJob:
        request = self._get_request(configs, urls)
        return self.start_job(request)

    def start_job(self, request_body: Any) -> BatchJob:
        """Start a batch job.

        Args:
            request_body (Any): JSON request body to be passed to the batch API.

        Raises:
            HumeClientError: If the batch job fails to start.

        Returns:
            BatchJob: A `BatchJob` that wraps the batch computation.
        """
        endpoint = (f"{self._api_base_url}/{self._api_version}/{ApiType.BATCH.value}/jobs"
                    f"?apikey={self._api_key}")
        response = requests.post(endpoint, json=request_body)

        try:
            body = response.json()
        except json.decoder.JSONDecodeError:
            # pylint: disable=raise-missing-from
            raise HumeClientError(f"Failed batch request: {response.text}")

        if "job_id" not in body:
            if "fault" in body and "faultstring" in body["fault"]:
                fault_string = body["fault"]["faultstring"]
                raise HumeClientError(f"Could not start batch job: {fault_string}")
            raise HumeClientError("Unexpected error when starting batch job")

        return BatchJob(self, body["job_id"])

    @classmethod
    def _get_request(cls, configs: List[JobConfigBase], urls: List[str]) -> Dict[str, Any]:
        model_requests = {}
        for config in configs:
            model_requests[config.get_model_type().value] = config.serialize()

        return {
            "models": model_requests,
            "urls": urls,
        }
